<?php

/**
 * Additional functions used in classes and another subclasses
 */
class Permalink_Manager_Post_URI_Functions extends Permalink_Manager_Class {

  public function __construct() {
		add_filter( '_get_page_link', array($this, 'custom_post_permalinks'), 999, 2);
		add_filter( 'page_link', array($this, 'custom_post_permalinks'), 999, 2);
		add_filter( 'post_link', array($this, 'custom_post_permalinks'), 999, 2);
		add_filter( 'post_type_link', array($this, 'custom_post_permalinks'), 999, 2);
		add_filter( 'permalink-manager-uris', array($this, 'exclude_homepage'), 999, 2);
    add_action( 'save_post', array($this, 'update_post_uri'), 10, 3 );
    add_action( 'wp_trash_post', array($this, 'remove_post_uri'), 10, 3 );
  }

  /**
	 * Change permalinks for posts, pages & custom post types
	 */
	function custom_post_permalinks($permalink, $post) {
		global $wp_rewrite, $permalink_manager_uris;

		$post = (is_integer($post)) ? get_post($post) : $post;
		$post_type = $post->post_type;

		// Do not change permalink of frontpage
		if(get_option('page_on_front') == $post->ID) { return $permalink; }
		if(isset($permalink_manager_uris[$post->ID])) $permalink = home_url('/') . $permalink_manager_uris[$post->ID];

		return $permalink;
	}

  /**
   * Display the permalink in a better way
   */
  static function get_correct_permalink($id) {
    global $permalink_manager_uris;
    $permalink = isset($permalink_manager_uris[$id]) ? home_url('/') . $permalink_manager_uris[$id] : get_permalink($id);

  	return $permalink;
  }

  /**
   * Check if the provided slug is unique and then update it with SQL query.
   */
  static function update_slug_by_id($slug, $id) {
    global $wpdb;

    // Update slug and make it unique
    $slug = (empty($slug)) ? sanitize_title(get_the_title($id)) : $slug;
    $new_slug = wp_unique_post_slug($slug, $id, get_post_status($id), get_post_type($id), null);
    $wpdb->query("UPDATE $wpdb->posts SET post_name = '$new_slug' WHERE ID = '$id'");

    return $new_slug;
  }

  /**
   * Get the active URI
   */
  static function get_post_uri($post_id) {
    global $permalink_manager_uris;

    // Check if input is post object
    $post_id = (isset($post_id->ID)) ? $post_id->ID : $post_id;

    $final_uri = isset($permalink_manager_uris[$post_id]) ? $permalink_manager_uris[$post_id] : self::get_default_post_uri($post_id);
    return $final_uri;
  }

  /**
   * Get the default (not overwritten by the user) or native URI (unfiltered)
   */
  static function get_default_post_uri($post, $native_uri = false) {
    global $permalink_manager_options, $permalink_manager_uris, $permalink_manager_permastructs;

    // Load all bases & post
    $post = is_object($post) ? $post : get_post($post);
    $post_id = $post->ID;
    $post_type = $post->post_type;
    $post_name = $post->post_name;

    // Get the permastruct
    $default_permastruct = Permalink_Manager_Helper_Functions::get_default_permastruct($post_type);

    if($native_uri) {
      $permastruct = $default_permastruct;
    } else if($permalink_manager_permastructs) {
      $permastruct = isset($permalink_manager_permastructs[$post_type]) ? $permalink_manager_permastructs[$post_type] : $default_permastruct;
    } else {
      $permastruct = isset($permalink_manager_options['base-editor'][$post_type]) ? $permalink_manager_options['base-editor'][$post_type] : $default_permastruct;
    }
    $default_base = (!empty($permastruct)) ? trim($permastruct, '/') : "";

    // 1A. Get the date
    $date = explode(" ",date('Y m d H i s', strtotime($post->post_date)));

    // 1B. Get the category slug (if needed)
    $category = '';
    if ( strpos($default_base, '%category%') !== false ) {

      // I. Try to use Yoast SEO Primary Term
      $category = (Permalink_Manager_Helper_Functions::get_primary_term($post->ID, 'category'));

      // II. Get the first assigned category
      if(empty($category)) {
        $cats = get_the_category($post->ID);
        if ($cats) {
          usort($cats, '_usort_terms_by_ID'); // order by ID
          $category_object = apply_filters( 'post_link_category', $cats[0], $cats, $post );
          $category_object = get_term( $category_object, 'category' );
          $category = $category_object->slug;
          if ( $parent = $category_object->parent )
            $category = get_category_parents($parent, false, '/', true) . $category;
        }
      }

      // III. Show default category in permalinks, without having to assign it explicitly
      if(empty($category)) {
        $default_category = get_term( get_option( 'default_category' ), 'category' );
        $category = is_wp_error( $default_category ) ? '' : $default_category->slug;
      }
    }

    // 1C. Get the author (if needed)
    $author = '';
    if ( strpos($default_base, '%author%') !== false ) {
      $authordata = get_userdata($post->post_author);
      $author = $authordata->user_nicename;
    }

    // 2. Fix for hierarchical CPT (start)
    $full_slug = get_page_uri($post);
    $post_type_tag = Permalink_Manager_Helper_Functions::get_post_tag($post_type);

    // 3A. Do the replacement (post tag is removed now to enable support for hierarchical CPT)
    $tags = array('%year%', '%monthnum%', '%day%', '%hour%', '%minute%', '%second%', '%post_id%', '%category%', '%author%', $post_type_tag);
    $replacements = array($date[0], $date[1], $date[2], $date[3], $date[4], $date[5], $post->ID, $category, $author, '');
    $default_uri = str_replace($tags, $replacements, "{$default_base}/{$full_slug}");

    // 3B. Replace custom taxonomies
    $terms = get_taxonomies( array('public' => true, '_builtin' => false), 'names', 'and' );
    $taxonomies = $terms;
    if ( $taxonomies ) {
      foreach($taxonomies as $taxonomy) {
        // A. Try to use Yoast SEO Primary Term
        $category = (Permalink_Manager_Helper_Functions::get_primary_term($post->ID, $taxonomy));

        // B. Get the first assigned term to this taxonomy
        if(empty($replacement)) {
          $terms = wp_get_object_terms($post->ID, $taxonomy);
          $replacement = (!is_wp_error($terms) && !empty($terms) && is_object($terms[0])) ? $terms[0]->slug : "";
        }

        // Do the replacement
        $default_uri = ($replacement) ? str_replace("%{$taxonomy}%", $replacement, $default_uri) : $default_uri;
      }
    }

    $default_uri = preg_replace('/\s+/', '', $default_uri);
    $default_uri = str_replace('//', '/', $default_uri);
    $default_uri = trim($default_uri, "/");

    return $default_uri;
  }

  /**
   * The homepage should not use URI
   */
  function exclude_homepage($uris) {
    // Find the homepage URI
    $homepage_id = get_option('page_on_front');
    if(isset($uris[$homepage_id])) { unset($uris[$homepage_id]); }

    return $uris;
  }

  /**
   * Find & replace (bulk action)
   */
   static function posts_find_and_replace() {
     global $wpdb, $permalink_manager_uris;

     // Reset variables
     $updated_slugs_count = 0;
     $updated_array = array();
     $alert_type = $alert_content = $errors = '';

     // Prepare default variables from $_POST object
     $old_string = esc_sql($_POST['old_string']);
     $new_string = esc_sql($_POST['new_string']);
     $mode = isset($_POST['mode']) ? $_POST['mode'] : array('both');
     $post_types_array = ($_POST['post_types']);
     $post_statuses_array = ($_POST['post_statuses']);
     $post_types = implode("', '", $post_types_array);
     $post_statuses = implode("', '", $post_statuses_array);

     // Save the rows before they are updated to an array
     $posts_to_update = $wpdb->get_results("SELECT post_title, post_name, ID FROM {$wpdb->posts} WHERE post_status IN ('{$post_statuses}') AND post_type IN ('{$post_types}')", ARRAY_A);

     // Now if the array is not empty use IDs from each subarray as a key
     if($posts_to_update && empty($errors)) {
       foreach ($posts_to_update as $row) {

         // Prepare variables
         $old_post_name = $row['post_name'];
         $native_uri = self::get_default_post_uri($row['ID'], true);
         $default_uri = self::get_default_post_uri($row['ID']);
         $old_uri = (isset($permalink_manager_uris[$row['ID']])) ? $permalink_manager_uris[$row['ID']] : $default_uri;
         $old_slug = (strpos($old_uri, '/') !== false) ? substr($old_uri, strrpos($old_uri, '/') + 1) : $old_uri;
         $old_base = (strpos($old_uri, '/') !== false) ? substr($old_uri, 0, strrpos( $old_uri, '/') ) : '';

         // Process URI & slug
         $new_slug = str_replace($old_string, $new_string, $old_slug);
         $new_base = str_replace($old_string, $new_string, $old_base);
         $new_uri = (in_array($mode, array('both'))) ? trim("{$new_base}/{$new_slug}", "/") : trim("{$old_base}/{$new_slug}", "/");
         $new_post_name = (in_array($mode, array('post_names'))) ? str_replace($old_string, $new_string, $old_post_name) : $old_post_name; // Post name is changed only in first mode

         //print_r("{$old_uri} - {$new_uri} - {$native_uri} - {$default_uri} \n");

         // Check if native slug should be changed
         if(in_array($mode, array('post_names')) && ($old_post_name != $new_post_name)) {
           self::update_slug_by_id($new_post_name, $row['ID']);
         }

         if(($old_uri != $new_uri) || ($old_post_name != $new_post_name)) {
           $permalink_manager_uris[$row['ID']] = $new_uri;
           $updated_array[] = array('post_title' => $row['post_title'], 'ID' => $row['ID'], 'old_uri' => $old_uri, 'new_uri' => $new_uri, 'old_slug' => $old_post_name, 'new_slug' => $new_post_name);
           $updated_slugs_count++;
         }

         // Do not store default values
         if(isset($permalink_manager_uris[$row['ID']]) && ($new_uri == $native_uri)) {
           unset($permalink_manager_uris[$row['ID']]);
         }
       }

       // Filter array before saving
       $permalink_manager_uris = array_filter($permalink_manager_uris);
       update_option('permalink-manager-uris', $permalink_manager_uris);

       $output = array('updated' => $updated_array, 'updated_count' => $updated_slugs_count);
       wp_reset_postdata();
     }

     return ($output) ? $output : "";
   }

   /**
    * Regenerate slugs & bases (bulk action)
    */
   static function posts_regenerate_all_permalinks() {
     global $wpdb, $permalink_manager_uris, $permalink_manager_permastructs;

     // Setup needed variables
     $updated_slugs_count = 0;
     $updated_array = array();
     $alert_type = $alert_content = $errors = '';

     $post_types_array = ($_POST['post_types']) ? ($_POST['post_types']) : '';
     $post_statuses_array = ($_POST['post_statuses']) ? $_POST['post_statuses'] : '';
     $post_types = implode("', '", $post_types_array);
     $post_statuses = implode("', '", $post_statuses_array);
     $mode = isset($_POST['mode']) ? $_POST['mode'] : 'both';

     // Save the rows before they are updated to an array
     $posts_to_update = $wpdb->get_results("SELECT post_title, post_name, post_type, ID FROM {$wpdb->posts} WHERE post_status IN ('{$post_statuses}') AND post_type IN ('{$post_types}')", ARRAY_A);

     // Now if the array is not empty use IDs from each subarray as a key
     if($posts_to_update && empty($errors)) {
       foreach ($posts_to_update as $row) {
         $updated = 0;

         // Prepare variables
         $old_post_name = $row['post_name'];
         $native_uri = self::get_default_post_uri($row['ID'], true);
         $default_uri = self::get_default_post_uri($row['ID']);
         $old_uri = isset($permalink_manager_uris[$row['ID']]) ? trim($permalink_manager_uris[$row['ID']], "/") : $native_uri;
         $old_slug = (strpos($old_uri, '/') !== false) ? substr($old_uri, strrpos($old_uri, '/') + 1) : $old_uri;
         $correct_slug = sanitize_title($row['post_title']);

         // Process URI & slug
         $new_slug = wp_unique_post_slug($correct_slug, $row['ID'], get_post_status($row['ID']), get_post_type($row['ID']), null);
         $new_post_name = (in_array($mode, array('post_names'))) ? $new_slug : $old_post_name; // Post name is changed only in first mode
         $new_uri = (in_array($mode, array('both'))) ? $default_uri : str_replace($old_slug, $new_slug, $old_uri);

         //print_r("{$old_uri} - {$new_uri} - {$native_uri} - {$default_uri} \n");

         // Check if native slug should be changed
         if(in_array($mode, array('post_names')) && ($old_post_name != $new_post_name)) {
           self::update_slug_by_id($new_post_name, $row['ID']);
         }

         if(($old_uri != $new_uri) || ($old_post_name != $new_post_name)) {
           $permalink_manager_uris[$row['ID']] = $new_uri;
           $updated_array[] = array('post_title' => $row['post_title'], 'ID' => $row['ID'], 'old_uri' => $old_uri, 'new_uri' => $new_uri, 'old_slug' => $old_post_name, 'new_slug' => $new_post_name);
           $updated_slugs_count++;
         }

         // Do not store default values
         if(isset($permalink_manager_uris[$row['ID']]) && ($new_uri == $native_uri)) {
           unset($permalink_manager_uris[$row['ID']]);
         }
       }

       // Filter array before saving
       $permalink_manager_uris = array_filter($permalink_manager_uris);
       update_option('permalink-manager-uris', $permalink_manager_uris);

       $output = array('updated' => $updated_array, 'updated_count' => $updated_slugs_count);
       wp_reset_postdata();
     }

     return (!empty($output)) ? $output : "";
   }

  /**
   * Update all slugs & bases (bulk action)
   */
  static public function posts_update_all_permalinks() {
    global $permalink_manager_uris;

    // Setup needed variables
    $updated_slugs_count = 0;
    $updated_array = array();

    $old_uris = $permalink_manager_uris;
    $new_uris = isset($_POST['uri']) ? $_POST['uri'] : array();

    // Double check if the slugs and ids are stored in arrays
    if (!is_array($new_uris)) $new_uris = explode(',', $new_uris);

    if (!empty($new_uris)) {
      foreach($new_uris as $id => $new_uri) {
        // Prepare variables
        $this_post = get_post($id);
        $updated = '';

        // Get default & native URL
        $native_uri = self::get_default_post_uri($id, true);
        $default_uri = self::get_default_post_uri($id);

        $old_uri = isset($old_uris[$id]) ? trim($old_uris[$id], "/") : $native_uri;

        // Process new values - empty entries will be treated as default values
        $new_uri = preg_replace('/\s+/', '', $new_uri);
        $new_uri = (!empty($new_uri)) ? trim($new_uri, "/") : $default_uri;
        $new_slug = (strpos($new_uri, '/') !== false) ? substr($new_uri, strrpos($new_uri, '/') + 1) : $new_uri;

        //print_r("{$old_uri} - {$new_uri} - {$native_uri} - {$default_uri} \n");

        // Do not store native URIs
        if($new_uri == $native_uri) {
          unset($old_uris[$id]);
        }

        if($new_uri != $old_uri) {
          $old_uris[$id] = $new_uri;
          $updated_array[] = array('post_title' => get_the_title($id), 'ID' => $id, 'old_uri' => $old_uri, 'new_uri' => $new_uri);
          $updated_slugs_count++;
        }

      }

      // Filter array before saving & append the global
      $old_uris = $permalink_manager_uris = array_filter($old_uris);
      update_option('permalink-manager-uris', $old_uris);

      $output = array('updated' => $updated_array, 'updated_count' => $updated_slugs_count);
    }

    return ($output) ? $output : "";
  }

  /**
	 * Update URI from "Edit Post" admin page
	 */
	function update_post_uri($post_id, $post, $update) {
    global $permalink_manager_uris;

		// Ignore trashed items
		if($post->post_status == 'trash') return;

		// Fix for revisions
		$is_revision = wp_is_post_revision($post_id);
		$post_id = ($is_revision) ? $is_revision : $post_id;
		$post = get_post($post_id);

		$native_uri = self::get_default_post_uri($post, true);
		$old_uri = (isset($permalink_manager_uris[$post->ID])) ? $permalink_manager_uris[$post->ID] : $native_uri;
		$new_uri = '';

		// Check if user changed URI (available after post is saved)
		if(isset($_POST['custom_uri'])) {
			$new_uri = trim($_POST['custom_uri'], "/");
		}

		// A little hack (if user removes whole URI from input) ...
		$new_uri = ($new_uri) ? $new_uri : self::get_post_uri($post);

		// Do not store default values
		if(isset($permalink_manager_uris[$post->ID]) && ($new_uri == $native_uri)) {
			unset($permalink_manager_uris[$post->ID]);
		}
    // Save only changed URIs
    else if (($new_uri != $native_uri) && ($new_uri != $old_uri)) {
			$permalink_manager_uris[$post->ID] = $new_uri;
		}

		update_option('permalink-manager-uris', $permalink_manager_uris);
	}

	/**
	 * Remove URI from options array after post is moved to the trash
	 */
	function remove_post_uri($post_id) {
		global $permalink_manager_uris;

		// Check if the custom permalink is assigned to this post
		if(isset($permalink_manager_uris[$post_id])) {
			unset($permalink_manager_uris[$post_id]);
		}

		update_option('permalink-manager-uris', $permalink_manager_uris);
	}

  /**
   * Remove URI from options array after post is moved to the trash
   */
  function clear_uris($post_id) {
    $uris = $this->permalink_manager_uris;

    foreach($uris as $post_id => $uri) {
      $post_status = get_post_status($post_id);
      if(in_array($post_status, array('auto-draft', 'trash', ''))) {
        unset($uris[$post_id]);
      }
    }

    update_option('permalink-manager-uris', $uris);
  }

}

?>
